% This function is for extracting data information from xml data of spots
% properties extracted from Trackmate imagej plugin for particle detection

% Due to different versions of Trackmate plugin from updating at around
% start of 2022,indexes for several variables have been changed. If the 
% output values are not reasonable, be sure to rerun the program by 
% commenting and uncommenting of above lines of code: 
% line 52-54 -- T(i) frame information
% line 61-69 -- MeanInt, MaxInt, TotInt Droplet mean/max/total intensity
% information

function [MeanInt,MaxInt,TotInt,spotN_T] = Read_TrackMatedata_xml(filename_xml)

% %% Select and read the xml file exported from TrackMate ImageJ tool
% disp('Please select the particle frame file generated by TrackMate.');
% [filename_xml,path] = uigetfile('.xml');
% addpath(path)

check = exist('parseXML.m','file'); % Check the existence of parseXML.m file for parsing .xml file to structure variable
if check ~= 2
    error('Cannot find parseXML.m in the trajectory, need to copy the function into the trajectory')
end

file_content = parseXML(filename_xml);

%% Extract information from structural variable read from .xml file
frame_info = file_content.Children(4).Children(4).Children;
frame_index = find(strcmp({frame_info.Name},{'SpotsInFrame'}));
frame_N = length(frame_index);

T = zeros(1, frame_N); %Time course of each frame
spotN_T = zeros(1, frame_N); %Number of detected droplets in each frame
MeanInt_T = zeros(1, frame_N); %Mean intensity of all droplets in each frame
SemMeanInt_T = zeros(1, frame_N); %Standard error of mean for mean droplet intensity in each frame
MaxInt_T = zeros(1, frame_N); %Max intensity of all droplets in each frame
StdMaxInt_T = zeros(1, frame_N); %Standard deviation of max droplet intensity in each frame
TotInt_T = zeros(1, frame_N); %Total intensity of all droplets in each frame
SemTotInt_T = zeros(1, frame_N); %Standard error of mean for total droplet intensity in each frame

MeanInt = cell(1, frame_N); %Cell array of mean intensity of all droplets in every frame (a.u.)
MaxInt = cell(1, frame_N); %Cell array of max intensity of all droplets in every frame (a.u.)
TotInt = cell(1, frame_N); %Cell array of total intensity of all droplets in every frame (a.u.)


for i = 1:frame_N
    spot_info = frame_info(frame_index(i)).Children;
    if ~isempty(spot_info)
        spot_index = find(strcmp({spot_info.Name},{'Spot'}));
        spot_N = length(spot_index);
        spotN_T(i) = spot_N;
        
        % New Trackmate index
        T(i) = str2double(spot_info(spot_index(1)).Attributes(8).Value); %Frame time information ('position_T')(s)
%         % Old Trackmate index
%         T(i) = str2double(spot_info(spot_index(1)).Attributes(10).Value);
        
        MeanInt_temp = zeros(1,spot_N); %Mean intensity from each droplet identified (a.u.)
        MaxInt_temp = zeros(1,spot_N); %Max intensity from each droplet identified (a.u.)
        TotInt_temp = zeros(1,spot_N); %Total intensity from each droplet identified (a.u.)
        for j = 1:spot_N
           % New Trackmate index:
            MeanInt_temp(j) = str2double(spot_info(spot_index(j)).Attributes(5).Value); %Mean_intensity information ('MEAN_INTENSITY')
            MaxInt_temp(j) = str2double(spot_info(spot_index(j)).Attributes(4).Value); %Max_intensity information ('MAX_INTENSITY')        
            TotInt_temp(j) = str2double(spot_info(spot_index(j)).Attributes(16).Value); %Total_intensity information ('TOTAL_INTENSITY')
            
%             % Old Trackmate index:
%             MeanInt_temp(j) = str2double(spot_info(spot_index(j)).Attributes(7).Value); %Mean_intensity information ('MEAN_INTENSITY')
%             MaxInt_temp(j) = str2double(spot_info(spot_index(j)).Attributes(6).Value); %Max_intensity information ('MAX_INTENSITY')        
%             TotInt_temp(j) = str2double(spot_info(spot_index(j)).Attributes(18).Value); %Total_intensity information ('TOTAL_INTENSITY')
        end
        MeanInt_T(i) = mean(MeanInt_temp);
        SemMeanInt_T(i) = std(MeanInt_temp)/sqrt(spot_N);
        MaxInt_T(i) = mean(MaxInt_temp);
        StdMaxInt_T(i) = std(MaxInt_temp);
        TotInt_T(i) = mean(TotInt_temp);
        SemTotInt_T(i) = std(TotInt_temp)/sqrt(spot_N);

    %Save related information (Mean intensity, max intensity, total intensity) of all droplets in every time frame into the cell array
        MeanInt{i} = MeanInt_temp;
        MaxInt{i} = MaxInt_temp; 
        TotInt{i} = TotInt_temp;
        
    else
        T(i) = str2double(frame_info(frame_index(i)).Attributes.Value); %Frame time information ('position_T')(s)
        spotN_T(i) = 0;
        MeanInt{i} = 0;
        MaxInt{i} = 0; 
        TotInt{i} = 0;
    end
   
end

% save ([filename_xml,'.mat'],'TotInt','MaxInt','MeanInt')
%% Plot the Droplet number, Droplet mean intensity, Droplet estimated Diameter
% figure('Name',filename_xml)
% plot(T,spotN_T,'.-')
% xlabel('Time/s')
% ylabel('Detected droplet number')
% 
% figure('Name',filename_xml)
% plot(T,MeanInt_T)  
% hold on
% errorbar(T,MeanInt_T, StdMeanInt_T,'*')
% xlabel('Time/s')
% ylabel('Droplet mean intensity/a.u.')
% 
% figure('Name',filename_xml)
% plot(T,MaxInt_T)
% hold on
% errorbar(T,MaxInt_T, StdMaxInt_T,'*')
% xlabel('Time/s')
% ylabel('Droplet max intensity/a.u.')
% 
% figure('Name',filename_xml)
% plot(T,TotInt_T,'o-')
% hold on 
% errorbar(T,TotInt_T,SemTotInt_T,'*')
% xlabel('Time/s')
% ylabel('Total droplet intensity in the frame/a.u.')
% 
% figure('Name',filename_xml)
% plot(T,EstD_T)  
% hold on
% errorbar(T,EstD_T, SemEstD_T,'*')
% xlabel('Time/s')
% ylabel('Estimated droplet diameter \mum')

% end
